@@ -303,6 +303,157 @@ def test_request_cookie__set_item() -> None:
         req.cookies["my"] = "value"
 
 
+def test_request_cookies_with_special_characters() -> None:
+    """Test that cookies with special characters in names are accepted.
+
+    This tests the fix for issue #2683 where cookies with special characters
+    like {, }, / in their names would cause a 500 error. The fix makes the
+    cookie parser more tolerant to handle real-world cookies.
+    """
+    # Test cookie names with curly braces (e.g., ISAWPLB{DB45DF86-F806-407C-932C-D52A60E4019E})
+    headers = CIMultiDict(COOKIE="{test}=value1; normal=value2")
+    req = make_mocked_request("GET", "/", headers=headers)
+    # Both cookies should be parsed successfully
+    assert req.cookies == {"{test}": "value1", "normal": "value2"}
+
+    # Test cookie names with forward slash
+    headers = CIMultiDict(COOKIE="test/name=value1; valid=value2")
+    req = make_mocked_request("GET", "/", headers=headers)
+    assert req.cookies == {"test/name": "value1", "valid": "value2"}
+
+    # Test cookie names with various special characters
+    headers = CIMultiDict(
+        COOKIE="test{foo}bar=value1; test/path=value2; normal_cookie=value3"
+    )
+    req = make_mocked_request("GET", "/", headers=headers)
+    assert req.cookies == {
+        "test{foo}bar": "value1",
+        "test/path": "value2",
+        "normal_cookie": "value3",
+    }
+
+
+def test_request_cookies_real_world_examples() -> None:
+    """Test handling of real-world cookie examples from issue #2683."""
+    # Example from the issue: ISAWPLB{DB45DF86-F806-407C-932C-D52A60E4019E}
+    headers = CIMultiDict(
+        COOKIE="ISAWPLB{DB45DF86-F806-407C-932C-D52A60E4019E}=val1; normal_cookie=val2"
+    )
+    req = make_mocked_request("GET", "/", headers=headers)
+    # All cookies should be parsed successfully
+    assert req.cookies == {
+        "ISAWPLB{DB45DF86-F806-407C-932C-D52A60E4019E}": "val1",
+        "normal_cookie": "val2",
+    }
+
+    # Multiple cookies with special characters
+    headers = CIMultiDict(
+        COOKIE="{cookie1}=val1; cookie/2=val2; cookie[3]=val3; cookie(4)=val4"
+    )
+    req = make_mocked_request("GET", "/", headers=headers)
+    assert req.cookies == {
+        "{cookie1}": "val1",
+        "cookie/2": "val2",
+        "cookie[3]": "val3",
+        "cookie(4)": "val4",
+    }
+
+
+def test_request_cookies_edge_cases() -> None:
+    """Test edge cases for cookie parsing."""
+    # Empty cookie value
+    headers = CIMultiDict(COOKIE="test=; normal=value")
+    req = make_mocked_request("GET", "/", headers=headers)
+    assert req.cookies == {"test": "", "normal": "value"}
+
+    # Cookie with quoted value
+    headers = CIMultiDict(COOKIE='test="quoted value"; normal=unquoted')
+    req = make_mocked_request("GET", "/", headers=headers)
+    assert req.cookies == {"test": "quoted value", "normal": "unquoted"}
+
+
+def test_request_cookies_no_500_error() -> None:
+    """Test that cookies with special characters don't cause 500 errors.
+
+    This specifically tests that issue #2683 is fixed - previously cookies
+    with characters like { } would cause CookieError and 500 responses.
+    """
+    # This cookie format previously caused 500 errors
+    headers = CIMultiDict(COOKIE="ISAWPLB{DB45DF86-F806-407C-932C-D52A60E4019E}=test")
+
+    # Should not raise any exception when accessing cookies
+    req = make_mocked_request("GET", "/", headers=headers)
+    cookies = req.cookies  # This used to raise CookieError
+
+    # Verify the cookie was parsed successfully
+    assert "ISAWPLB{DB45DF86-F806-407C-932C-D52A60E4019E}" in cookies
+    assert cookies["ISAWPLB{DB45DF86-F806-407C-932C-D52A60E4019E}"] == "test"
+
+
+def test_request_cookies_quoted_values() -> None:
+    """Test that quoted cookie values are handled consistently.
+
+    This tests the fix for issue #5397 where quoted cookie values were
+    handled inconsistently based on whether domain attributes were present.
+    The new parser should always unquote cookie values consistently.
+    """
+    # Test simple quoted cookie value
+    headers = CIMultiDict(COOKIE='sess="quoted_value"')
+    req = make_mocked_request("GET", "/", headers=headers)
+    # Quotes should be removed consistently
+    assert req.cookies == {"sess": "quoted_value"}
+
+    # Test quoted cookie with semicolon in value
+    headers = CIMultiDict(COOKIE='data="value;with;semicolons"')
+    req = make_mocked_request("GET", "/", headers=headers)
+    assert req.cookies == {"data": "value;with;semicolons"}
+
+    # Test mixed quoted and unquoted cookies
+    headers = CIMultiDict(
+        COOKIE='quoted="value1"; unquoted=value2; also_quoted="value3"'
+    )
+    req = make_mocked_request("GET", "/", headers=headers)
+    assert req.cookies == {
+        "quoted": "value1",
+        "unquoted": "value2",
+        "also_quoted": "value3",
+    }
+
+    # Test escaped quotes in cookie value
+    headers = CIMultiDict(COOKIE=r'escaped="value with \" quote"')
+    req = make_mocked_request("GET", "/", headers=headers)
+    assert req.cookies == {"escaped": 'value with " quote'}
+
+    # Test empty quoted value
+    headers = CIMultiDict(COOKIE='empty=""')
+    req = make_mocked_request("GET", "/", headers=headers)
+    assert req.cookies == {"empty": ""}
+
+
+def test_request_cookies_with_attributes() -> None:
+    """Test that cookie attributes don't affect value parsing.
+
+    Related to issue #5397 - ensures that the presence of domain or other
+    attributes doesn't change how cookie values are parsed.
+    """
+    # Cookie with domain attribute - quotes should still be removed
+    headers = CIMultiDict(COOKIE='sess="quoted_value"; Domain=.example.com')
+    req = make_mocked_request("GET", "/", headers=headers)
+    assert req.cookies == {"sess": "quoted_value"}
+
+    # Cookie with multiple attributes
+    headers = CIMultiDict(COOKIE='token="abc123"; Path=/; Secure; HttpOnly')
+    req = make_mocked_request("GET", "/", headers=headers)
+    assert req.cookies == {"token": "abc123"}
+
+    # Multiple cookies with different attributes
+    headers = CIMultiDict(
+        COOKIE='c1="v1"; Domain=.example.com; c2="v2"; Path=/api; c3=v3; Secure'
+    )
+    req = make_mocked_request("GET", "/", headers=headers)
+    assert req.cookies == {"c1": "v1", "c2": "v2", "c3": "v3"}
+
+
 def test_match_info() -> None:
     req = make_mocked_request("GET", "/")
     assert req._match_info is req.match_info
